---
title: init方法
---
这节分析这一行代码的作用
```php
Hyperf\Di\ClassLoader::init();
```
`init`方法有三个可选参数，默认是不传参数进来
```php
public static function init(?string $proxyFileDirPath = null, ?string $configDir = null, ?ScanHandlerInterface $handler = null): void
{
    // 代理文件目录
    if (! $proxyFileDirPath) {
        $proxyFileDirPath = BASE_PATH . '/runtime/container/proxy/';
    }

    // 配置文件目录
    if (! $configDir) {
        $configDir = BASE_PATH . '/config/';
    }

    // 扫描器，用来扫描类文件并生成类映射
    // 利用 PHP 的 pcntl 扩展提供的功能来进行并发扫描，以加快扫描速度
    if (! $handler) {
        $handler = new PcntlScanHandler();
    }
    // 获取composer的自动加载器实例
    $composerLoader = Composer::getLoader();

    // 这就没啥说的， 如果.env文件存在，就加载.env文件
    // 使用vlucas/phpdotenv库
    if (file_exists(BASE_PATH . '/.env')) {
        static::loadDotenv();
    }

    // Scan by ScanConfig to generate the reflection class map
    $config = ScanConfig::instance($configDir);
    $composerLoader->addClassMap($config->getClassMap());

    $scanner = new Scanner($config, $handler);
    $composerLoader->addClassMap(
        $scanner->scan($composerLoader->getClassMap(), $proxyFileDirPath)
    );

    // Initialize Lazy Loader. This will prepend LazyLoader to the top of autoload queue.
    LazyLoader::bootstrap($configDir);
}

protected static function loadDotenv(): void
{
    $repository = RepositoryBuilder::createWithNoAdapters()
        ->addAdapter(Adapter\PutenvAdapter::class)
        ->immutable()
        ->make();

    Dotenv::create($repository, [BASE_PATH])->load();
}
```
### instance方法
看这行，`instance`方法创建一个`ScanConfig`实例，用于加载并解析目录下的配置文件，并生成类映射等扫描相关的配置信息。不理解可以往下看，
```php
$config = ScanConfig::instance($configDir);
```
进入该方法，
```php
public static function instance(string $configDir): self
{
    // 如果该实例存在，则直接返回该实例
    if (self::$instance) {
        return self::$instance;
    }

    $configDir = rtrim($configDir, '/');

    [$config, $serverDependencies, $cacheable] = static::initConfigByFile($configDir);
    // 这里返回该实例
    return self::$instance = new self(
        $cacheable,
        $configDir,
        $config['paths'] ?? [],
        $serverDependencies ?? [],
        $config['ignore_annotations'] ?? [],
        $config['global_imports'] ?? [],
        $config['collectors'] ?? [],
        $config['class_map'] ?? []
    );
}
```
#### initConfigByFile方法
配置的获取在该方法中，
```php
private static function initConfigByFile(string $configDir): array
{
    $config = [];
    $configFromProviders = [];
    $cacheable = false;
    // 该class是hyperf config包中带的
    if (class_exists(ProviderConfig::class)) {
        $configFromProviders = ProviderConfig::load();
    }

   //...
}
```
继续往下看，`ProviderConfig::load()`进入该方法，
```php
public static function load(): array
{
    // 如果未赋值，则进行赋值
    if (! static::$providerConfigs) {
        // 该行是从composer.json中获取配置信息，并返回数组
        // 这里会获得所有的服务提供者信息
        $providers = Composer::getMergedExtra('hyperf')['config'] ?? [];
        static::$providerConfigs = static::loadProviders($providers);
    }
    return static::$providerConfigs;
}
```
这里用到了[ConfigProvider 机制](https://hyperf.wiki/3.1/#/zh-cn/component-guide/configprovider)，参考官网讲解。
`Composer::getMergedExtra`获取`composer.json`文件中的配置信息，内容如下
![](https://oss.xiaokeaii.top/2024/composer::getMergedExtra.png)
打印获取的内容如下，是所有服务提供者的数组
![](https://oss.xiaokeaii.top/2024/provider.png)
接下来查看`loadProviders`方法
```php
protected static function loadProviders(array $providers): array
{
    $providerConfigs = [];
    // 循环所有服务提供者
    foreach ($providers as $provider) {
        // 如果是字符串，且存在该类，且该类存在__invoke方法
        if (is_string($provider) && class_exists($provider) && method_exists($provider, '__invoke')) {
            // 当通过方法调用类时，会触发__invoke方法
            // 所以(new $provider())()相当于执行服务提供者的__invoke方法
            $providerConfigs[] = (new $provider())();
        }
    }

    return static::merge(...$providerConfigs);
}
```
我们以`hyperf/config`包为例，打开目录下`ConfigProvider`类，打开`__invoke`方法，会发现，这里返回的是一个配置数组，代码如下，
```php
class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'dependencies' => [
                ConfigInterface::class => ConfigFactory::class,
            ],
            'aspects' => [
                ValueAspect::class,
            ],
            'listeners' => [
                RegisterPropertyHandlerListener::class,
            ],
        ];
    }
}
```
最终生成的`$providerConfigs`数组格式如下，每个服务提供者都会生成一个数组。
```php
array(22) {
  [0]=>
  array(5) {
    ["dependencies"]=>
    array(1) {
      ["Psr\SimpleCache\CacheInterface"]=>
      string(18) "Hyperf\Cache\Cache"
    }
    ["listeners"]=>
    array(1) {
      [0]=>
      string(36) "Hyperf\Cache\Listener\DeleteListener"
    }
    ["annotations"]=>
    array(1) {
      ["scan"]=>
      array(1) {
        ["collectors"]=>
        array(1) {
          [0]=>
          string(35) "Hyperf\Cache\CacheListenerCollector"
        }
      }
    }
    ["aspects"]=>
    array(5) {
      [0]=>
      string(35) "Hyperf\Cache\Aspect\CacheableAspect"
      [1]=>
      string(36) "Hyperf\Cache\Aspect\CacheAheadAspect"
      [2]=>
      string(36) "Hyperf\Cache\Aspect\CacheEvictAspect"
      [3]=>
      string(34) "Hyperf\Cache\Aspect\CachePutAspect"
      [4]=>
      string(35) "Hyperf\Cache\Aspect\FailCacheAspect"
    }
    ["publish"]=>
    array(1) {
      [0]=>
      array(4) {
        ["id"]=>
        string(6) "config"
        ["description"]=>
        string(21) "The config for cache."
        ["source"]=>
        string(74) "/data/project/hyperf-skeleton/vendor/hyperf/cache/src/../publish/cache.php"
        ["destination"]=>
        string(55) "/data/project/hyperf-skeleton/config/autoload/cache.php"
      }
    }
  }
  //...
}
```
最后再看`return static::merge(...$providerConfigs);`这一行，查看`static::merge`方法，该方法是将所有服务提供者的配置数组进行合并，最终返回一个数组。
```php
protected static function merge(...$arrays): array
{
    if (empty($arrays)) {
        return [];
    }
    // 递归地合并一个或多个数组,
    // 如果输入的数组中有相同的字符串键名，则这些值会被合并到一个数组中去
    $result = array_merge_recursive(...$arrays);
    // 如果存在dependencies key，则对依赖进行合并
    if (isset($result['dependencies'])) {
        $result['dependencies'] = [];
        foreach ($arrays as $item) {
            foreach ($item['dependencies'] ?? [] as $key => $value) {
                $depend = $result['dependencies'][$key] ?? null;
                if (! $depend instanceof PriorityDefinition) {
                    $result['dependencies'][$key] = $value;
                    continue;
                }

                if ($value instanceof PriorityDefinition) {
                    $depend->merge($value);
                }
            }
        }
    }
    // 返回合并后的配置数组
    return $result;
}
```
以上，`ProviderConfig::load()`方法走完。
往下看，以下是将config目录下配置与上面获取的配置进行合并，存在重复则配置文件优先
```php
 $serverDependencies = $configFromProviders['dependencies'] ?? [];
    if (file_exists($configDir . '/autoload/dependencies.php')) {
        $definitions = include $configDir . '/autoload/dependencies.php';
        $serverDependencies = array_replace($serverDependencies, $definitions ?? []);
    }

    $config = static::allocateConfigValue($configFromProviders['annotations'] ?? [], $config);

    // 这里是注解相关的配置，回来在分析
    if (file_exists($configDir . '/autoload/annotations.php')) {
        $annotations = include $configDir . '/autoload/annotations.php';
        $config = static::allocateConfigValue($annotations, $config);
    }

    // Load the config/config.php and merge the config
    if (file_exists($configDir . '/config.php')) {
        $configContent = include $configDir . '/config.php';
        $appEnv = $configContent['app_env'] ?? 'dev';
        $cacheable = value($configContent['scan_cacheable'] ?? $appEnv === 'prod');
        if (isset($configContent['annotations'])) {
            $config = static::allocateConfigValue($configContent['annotations'], $config);
        }
    }

    return [$config, $serverDependencies, $cacheable];
```
`instance`方法的三个返回值都已经清楚，接下来返回自身实例
```php
return self::$instance = new self(
    $cacheable,
    $configDir,
    $config['paths'] ?? [],
    $serverDependencies ?? [],
    $config['ignore_annotations'] ?? [],
    $config['global_imports'] ?? [],
    $config['collectors'] ?? [],
    $config['class_map'] ?? []
);
```
现在，`config`文件配置已经得到，
```php
$config = ScanConfig::instance($configDir);
```
分析下一行，
```php
// 将提供的类映射添加到 Composer 的自动加载器中
$composerLoader->addClassMap($config->getClassMap());
```
由上面配置文件可知，默认情况下，`class_map`为空。